<?php

/**
 * TargetPay model IDEAL
 *
 * @category    TargetMedia
 * @package     TargetPay model
 * @author      TargetMedia SvE
 */
ini_set ( 'display_errors', '1' );   

class Mage_Tp_Model_TpPayment extends Mage_Payment_Model_Method_Abstract
{
	
    // Main model code
    protected $_code = "tp";
    protected $_formBlockType = 'tp/form';  
       
    protected $_isGateway              = false;
    protected $_canAuthorize           = false;
    protected $_canCapture             = true;
    protected $_canCapturePartial      = false;
    protected $_canRefund              = false;
    protected $_canVoid                = false;
    protected $_canUseInternal         = false;
    protected $_canUseCheckout         = true;
    protected $_canUseForMultishipping = false;
    
    // Version
	private $_tp_version = '0.0.1';
    
    
	public function getCheckout() 
    {
		return Mage::getSingleton("checkout/session");
	}
	
	public function getQuote() 
    {
		return $this->getCheckout()->getQuote();
	}

	public function getOrderPlaceRedirectUrl() 
    {
		return Mage::getUrl("tp/tpPayment/redirect", array("_secure" => true));
	}
    
    public function getPayment()
    {
        return $this->getQuote()->getPayment();
    }


    // This is wehere we start the transaction
	public function startTransaction()
    {
		
        // only euro
		if ($this->getQuote()->getBaseCurrencyCode() != "EUR")
			Mage::throwException(Mage::helper("tp")->__("Currently only EUR payments are supported by the ideal module."));
        
        //Minimal 1 euro for IDEAL ( We get errors when we do less!)  
        if ($this->getQuote()->getGrandTotal() < 1)
            Mage::throwException(Mage::helper("tp")->__("The minimum amount is 1 EURO for this paymethod."));
        
        // Retrieve checkout    
		$checkout = $this->getCheckout();
        
        $bank_id = $this->getPayment()->cc_type;
    
		// Set parameters 
		$rtlo        = Mage::getStoreConfig("payment/tp/account_id");
		$notify_url  = Mage::getUrl("tp/tpPayment/notification", array("_secure" => true)) . "?type=initial";
		$cancel_url  = Mage::getUrl("checkout");
        $returnurl   = Mage::getUrl('tp/tpPayment/return');
		$storename   = Mage::getStoreConfig('system/store/name');
		$order_id    = $checkout->getLastRealOrderId();
		$amount      = intval($this->getQuote()->getGrandTotal() * 100);
		$currency 	 = "EUR";
        $language    = "Nl";
        
        // Bank id from select field
        $bank = $bank_id;
        $description = Mage::getStoreConfig("payment/tp/desc");  

        // Init ideal class
        $ideal = new iDEAL ($rtlo);
        $result = $ideal->GetLink ($bank, $description, $amount, $returnurl);
        
        if ( !$result )
        {
            $this->error = $result;
            Mage::throwException(Mage::helper("tp")->__("An error occured: " . $this->error));
        } 
        else 
        {
            $this->trxid = $ideal->trxid;
            $this->url = $ideal->url;
            setcookie ("trxid", $this->trxid, time()+86400);
        }
        
		return $this->url;
	}
    
    // We return from the bank. Lets validate..	
	public function updateOrder($order_id)
    {
		
        // Retrieve rtlo from config:)
        $rtlo  = Mage::getStoreConfig("payment/tp/account_id");
        $checkout = $this->getCheckout();
        
        // Order id
        $tr_id    = $checkout->getLastRealOrderId();
        
        //$tr_id = $this->getRequest()->getQuery('transactionid');
        $order = Mage::getSingleton('sales/order')->loadByIncrementId($tr_id);
        
        // If auto invoice is on in config?
		$autocreate_invoice = Mage::getStoreConfig("payment/tp/autocreate_invoice");
        
        //Init ideal and validate this transaction.
        $ideal = new iDEAL ($rtlo);
        $result = $ideal->CheckPayment ($order_id);
        $newState = NULL;
        $complete = false;
        
        // not payed
        if (!$result) 
        {
            $this->error = $ideal->error;
            $newState = Mage_Sales_Model_Order::STATE_CANCELED;
            //Mage::throwException(Mage::helper("tp")->__("An error occured: " . $this->error));
        } 
        else 
        {    
            // send an order e-mail to customer.
            $order = Mage::getSingleton('sales/order')->loadByIncrementId($order_id);
            $order->sendNewOrderEmail();
            $order->setEmailSent(true);
            $order->save();
            
            // Transaction accepted
            $this->trxid = $order_id;
            $complete = true;
            $newState = "completed";
        }  
    
        // create an invoice when the payment is completed
        if ($complete && $autocreate_invoice){
            $this->_saveInvoice($order);
        }
        
        // update the status
        if ($newState !== null && $newState != $order->getStatus()){
            $order->setState($newState, true, $result);
            $order->save();
        }
        
        if ( !$result  ){
            return false;
        }
        elseif ( $result == true ) {
		    return true;
        }
        

	}
	
	/**
	 *  Save invoice for order
	 *
	 *  @param    Mage_Sales_Model_Order $order
	 *  @return	  boolean Can save invoice or not
	 */
	protected function _saveInvoice(Mage_Sales_Model_Order $order){
		if ($order->canInvoice()) {
			$invoice = $order->prepareInvoice();
			$invoice->register()->capture();
			Mage::getModel('core/resource_transaction')
				->addObject($invoice)
				->addObject($invoice->getOrder())
				->save();
			return true;
		}
		return false;
	}
}

/*

Http request

*/

class httpRequest {
    var $response;

    //
    //    Do a HTTP POST request to $url, put the arguments in $arguments
    //
    //    E.g.
    //
    //    $http = new httpRequest();
    //    $http->httpRequest ("http://www.yoururl.nl", array("a" => $a, "b" => $b));
    //    echo $http->response;
    //

    function httpRequest ($url, $arguments) {
        if (substr($url,0,7)=="http://") {
            $url = substr($url,7);
            }

        list ($host,$uri) = explode("/",$url,2);
        $uri = "/$uri";

        $querystr = "";
        foreach ($arguments as $k => $v) {
            $querystr .= "&$k=".urlencode($v);
            }
        $querystr = substr($querystr, 1);
        $qlength = strlen($querystr);

        $remote = fsockopen($host, 80, $errno, $errstr, 30);

        $post =
            "POST $uri HTTP/1.0\r\n".
            "Host: $host\r\n".
            "Content-type: application/x-www-form-urlencoded\r\n".
            "Content-length: $qlength\r\n\r\n".
            "$querystr\r\n\r\n";

        if ($remote) {
              fputs($remote, $post);
              $response = "";
              while(!feof($remote)) {
                $a = fgets($remote,4096);
                $response .= $a;
                }
              } else {
            $this->errorHandler ("Connection failed $errno $errstr");
            }

        list ($header, $response) = explode("\r\n\r\n", $response, 2);
        list ($temp, $statuscode) = explode(" ", $header, 2);
        list ($statuscode) = explode("\r\n", $statuscode);

        if (substr($statuscode,0,3)=="200") {
            $this->response = $response;
            } else {
            $this->errorHandler ($statuscode);
            }
        }

    function errorHandler ($errorMessage) {
        die ($errorMessage);
        }

    }
    
    
class iDEAL {
    var $rtlo;          

    var $error;     

    var $callduration;    
    var $calltariff;    

    var $trxid;     
    var $url;        

    function iDEAL ($rtlo) {
        $this->rtlo = $rtlo;
    }

    function GetLink ($bank, $description, $amount, $returnurl, $currency="EUR", $language="nl") {
        $dataToSend = array(
            "rtlo" => $this->rtlo,
            "bank" => $bank,
            "description" => $description,
            "amount" => $amount,
            "returnurl" => $returnurl,
            "currency" => $currency,
            "language" => $language
            );

        $api = new httpRequest ("http://www.targetpay.nl/ideal/start", $dataToSend);
        list ($resultcode, $result) = explode(" ", $api->response,2);
        list ($trxid, $url) = explode("|", $result);

        if ($resultcode=="000000") {
            $this->trxid = $trxid;
            $this->url = $url;
            return true;
            } else {
            $this->error = $result;
            return false;
            }
        }

    function CheckPayment ($trxid, $once=1) {
        $dataToSend = array(
            "rtlo" => $this->rtlo,
            "trxid" => $trxid,
            "once" => $once
            );

        $api = new httpRequest ("http://www.targetpay.nl/ideal/check", $dataToSend);
        list ($resultcode, $result) = explode(" ", $api->response,2);
        
        if ($resultcode=="000000") {
            return true;
        } 
        else 
        {
            $this->error = $result;
            return false;
        }
       
    }


}

?>
